package vc.thinker.web;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import vc.thinker.config.DataReportQueueUtil;
import vc.thinker.config.entity.DataReportArrayQueue;
import vc.thinker.config.interceptor.ServiceLimit;
import vc.thinker.mqtt.forwardthinker.ThinkerProdProtocolClient;
import vc.thinker.response.SimpleResponse;
import vc.thinker.thirdpartyproxy.protocolimpls.centralAir.CentralAirProtocol;
import vc.thinker.thirdpartyproxy.protocolimpls.centralAir.dto.DataReportInfoDTO;
import vc.thinker.thirdpartyproxy.protocolimpls.centralAir.dto.DataReportParamDTO;
import vc.thinker.web.vo.TerminalDataReportVO;

import javax.validation.Valid;
import java.util.concurrent.ArrayBlockingQueue;


/**
 * @author HeTongHao
 * @since 2022/8/9 15:56
 */
@Api
@Slf4j
@RequestMapping("/")
@RestController
@RequiredArgsConstructor
public class CentralAirController {

    @Autowired
    private CentralAirProtocol centralAirProtocol;
    @Autowired
    private ThinkerProdProtocolClient thinkerProdProtocolClient;


    @ApiOperation("设备数据上报接口")
    @PostMapping("data-report")
    @ServiceLimit(key = "data-report")
    public SimpleResponse dataReport(@RequestBody @Valid TerminalDataReportVO reportVO) throws MqttException {
        SimpleResponse simpleResponse = new SimpleResponse();
        simpleResponse.setCode("200");
        //if (Objects.equals(reportVO.getProtocolCode(),ProtocolType.central_air_conditioning.getCode())){
            DataReportInfoDTO dto = DataReportInfoDTO.builder()
                    .protocolCode(reportVO.getProtocolCode())
                    //.deviceAddress(reportVO.getDeviceAddress())
                    .timestamp(reportVO.getTimestamp())
                    .data(JSONObject.parseObject(JSONObject.toJSONString(reportVO.getData()), DataReportParamDTO.class))
                    .build();
            this.centralAirProtocol.dataReport(dto);
            simpleResponse.setSuccess(true);
            simpleResponse.setMessage("处理成功");
            ArrayBlockingQueue<DataReportArrayQueue> queue = DataReportQueueUtil.getArrayBlockingQueue();
            for (DataReportArrayQueue obj : queue){
                DataReportArrayQueue poll = queue.poll();
                if (!ObjectUtils.isEmpty(poll)){
                    this.thinkerProdProtocolClient.reportBytes(poll.getTopic(),poll.getPayload());
                }

            }
            return simpleResponse;
        //}
        // simpleResponse.setSuccess(false);
        // simpleResponse.setMessage("处理失败");
        // return simpleResponse;
    }

}
